@@ -1,4 +1,4 @@
-# (c) 2011-2012 Dennis Kaarsemaker <dennis@kaarsemaker.net>
+# (c) 2011-2013 Dennis Kaarsemaker <dennis@kaarsemaker.net>
 # see COPYING for license details
 
 import os
@@ -30,7 +30,26 @@ except ImportError:
         PROTOCOL_TLSv1 = 3
         @staticmethod
         def wrap_socket(sock, *args, **kwargs):
-            return socket.ssl(sock)
+            return ssl(sock)
+
+        def __init__(self, sock):
+            self.sock = sock
+            self.sslsock = socket.ssl(sock)
+
+        def read(self, n=None):
+            if not n:
+                return self.sslsock.read()
+            return self.sslsock.read(n)
+
+        def write(self, data):
+            return self.sslsock.write(data)
+
+        def shutdown(self, what):
+            return self.sock.shutdown(what)
+
+        def close(self):
+            return self.sock.close()
+
 try:
     import xml.etree.cElementTree as etree
 except ImportError:
@@ -63,6 +82,9 @@ class IloError(Exception):
 class IloCommunicationError(IloError):
     pass
 
+# When we stop supporting ilo 1, 'User login name was not found' and 0x000a can
+# be removed. They should, as they cause IloLoginFailed in cases where login
+# did not fail, but e.g. get_user('nonexistent') was called
 class IloLoginFailed(IloError):
     possible_messages = ['User login name was not found', 'Login failed', 'Login credentials rejected']
     possible_codes = [0x005f, 0x000a]
@@ -71,6 +93,9 @@ class IloLoginFailed(IloError):
 class IloWarning(Warning):
     pass
 
+class IloTestWarning(Warning):
+    pass
+
 class Ilo(object):
     """Represents an iLO/iLO2/iLO3/RILOE II management interface on a
         specific host. A new connection using the specified login, password and
@@ -318,6 +343,12 @@ class Ilo(object):
                 raise IloCommunicationError("Communication with %s:%d failed: %s" % (self.hostname, self.port, str(e)))
 
         self._debug(1, "Received %d bytes" % len(data))
+        if self.protocol == ILO_LOCAL:
+            sock.stdout.close()
+            sock.wait()
+        elif sock.shutdown:
+            sock.shutdown(socket.SHUT_RDWR)
+            sock.close()
 
         # Stript out garbage from hponcfg
         if self.protocol == ILO_LOCAL:
@@ -419,6 +450,11 @@ class Ilo(object):
             # Can't return a dict
             retval = []
         for elt in element:
+            # There are some special tags
+            fname =  '_parse_%s_%s' % (element.tag.lower(), elt.tag.lower())
+            if hasattr(self, fname):
+                retval.update(getattr(self, fname)(elt))
+                continue
             key, val, unit = elt.tag.lower(), elt.get('VALUE', elt.get('value', None)), elt.get('UNIT', None)
             if val is None:
                 # HP is not best friends with consistency. Sometimes there are
@@ -521,30 +557,6 @@ class Ilo(object):
                 return process(self._element_to_dict(message))
         raise IloError("Expected tag '%s' not found" % "' or '".join(returntags))
 
-    def _info_tag2(self, infotype, tagname, returntag=None, key=None, process=lambda x: x):
-        root, inner = self._root_element(infotype, MODE='read')
-        etree.SubElement(inner, tagname)
-        if self.delayed:
-            self._processors.append([self._process_info_tag2, returntag or tagname, key, process])
-            return
-        header, message = self._request(root)
-        return self._process_info_tag2(message, returntag or tagname, key, process)
-
-    def _process_info_tag2(self, message, returntag, key, process):
-        message = message.find(returntag)
-
-        if key:
-            retval = {}
-        else:
-            retval = []
-        for elt in message:
-            elt = self._element_to_dict(elt)
-            if key:
-                retval[elt[key]] = elt
-            else:
-                retval.append(elt)
-        return process(retval)
-
     def _control_tag(self, controltype, tagname, returntag=None, attrib={}, elements=[], text=None):
         root, inner = self._root_element(controltype, MODE='write')
         inner = etree.SubElement(inner, tagname, **attrib)
@@ -584,7 +596,9 @@ class Ilo(object):
            The return value of call_delayed is a list of return values for
            individual methods that don't return None. This means that there may
            be fewer items returned than methods called as only `get_*` methods
-           return data"""
+           return data
+
+           Delayed calls only work on iLO 2 or newer"""
 
         if not self._elements:
             raise ValueError("No commands scheduled")
@@ -677,12 +691,20 @@ class Ilo(object):
 
     def get_all_users(self):
         """Get a list of all loginnames"""
-        return self._info_tag2('USER_INFO', 'GET_ALL_USERS', key='value',
-                process=lambda data: [x for x in data if x])
+        def process(data):
+            if isinstance(data, dict):
+                data = data.values()
+            return [x for x in data if x]
+
+        return self._info_tag('USER_INFO', 'GET_ALL_USERS', process=process)
 
     def get_all_user_info(self):
         """Get basic and authorization info of all users"""
-        return self._info_tag2('USER_INFO', 'GET_ALL_USER_INFO', key='user_login')
+        def process(data):
+            if isinstance(data, dict):
+                data = data.values()
+            return dict([(x['user_login'], x) for x in data])
+        return self._info_tag('USER_INFO', 'GET_ALL_USER_INFO', process=process)
 
     def get_cert_subject_info(self):
         """Get ssl certificate subject information"""
@@ -704,13 +726,100 @@ class Ilo(object):
                         else:
                             health[key].update(val)
                     data[category] = health
-                else:
-                    tag = 'label' in data[category][0] and 'label' or 'location'
-                    data[category] = dict([(x[tag], x) for x in data[category]])
+                    continue
+                elif isinstance(data[category], list):
+                    for tag in ('label', 'location'):
+                        if tag in data[category][0]:
+                            data[category] = dict([(x[tag], x) for x in data[category]])
+                            break
+                elif data[category] == '':
+                    data[category] = None
             return data
         return self._info_tag('SERVER_INFO', 'GET_EMBEDDED_HEALTH', 'GET_EMBEDDED_HEALTH_DATA',
                 process=process)
 
+    # Ok, special XML structures. Yay.
+    def _parse_get_embedded_health_data_drives(self, element):
+        ret = []
+        for bp in element:
+            if bp.tag != 'BACKPLANE':
+                raise IloError("Unexpected data returned: %s" % bp.tag)
+            backplane =  obj = {'drive_bays': {}}
+            ret.append(backplane)
+            for elt in bp:
+                if elt.tag == 'DRIVE_BAY':
+                    obj = {}
+                    backplane['drive_bays'][int(elt.get('VALUE'))] = obj
+                else:
+                    obj[elt.tag.lower()] = elt.get('VALUE')
+        return {'drives_backplanes': ret}
+
+    def _parse_get_embedded_health_data_memory(self, element):
+        ret = {}
+        for elt in element:
+            fname =  '_parse_%s_%s' % (element.tag.lower(), elt.tag.lower())
+            if hasattr(self, fname):
+                ret.update(getattr(self, fname)(elt))
+                continue
+            ret[elt.tag.lower()] = self._element_children_to_dict(elt)
+        return {element.tag.lower(): ret}
+    _parse_memory_memory_details_summary = _parse_get_embedded_health_data_memory
+
+    def _parse_memory_memory_details(self, element):
+        ret = {}
+        for elt in element:
+            if elt.tag not in ret:
+                ret[elt.tag] = {}
+            data = self._element_children_to_dict(elt)
+            ret[elt.tag]["socket %d" % data["socket"]] = data
+        return {element.tag.lower(): ret}
+
+    def _parse_get_embedded_health_data_nic_information(self, element):
+        ret = {}
+        for elt in element:
+            data = self._element_children_to_dict(elt)
+            ret[data['network_port']] = data
+        return {element.tag.lower(): ret}
+
+    def _parse_get_embedded_health_data_firmware_information(self, element):
+        ret = {}
+        for elt in element:
+            data = self._element_children_to_dict(elt)
+            ret[data['firmware_name']] = data['firmware_version']
+        return {element.tag.lower(): ret}
+
+    def _parse_get_embedded_health_data_storage(self, element):
+        ret = []
+        for ctrl in element:
+            data = {}
+            for elt in ctrl:
+                tag = elt.tag.lower()
+                if tag in ('drive_enclosure', 'logical_drive'):
+                    tag += 's'
+                    if tag not in data:
+                        data[tag] = []
+                    if tag == 'drive_enclosures':
+                        data[tag].append(self._element_children_to_dict(elt))
+                    else:
+                        data[tag].append(self._parse_logical_drive(elt))
+                else:
+                    data[tag] = elt.get('VALUE')
+            ret.append(data)
+        return {element.tag.lower(): ret}
+
+    def _parse_logical_drive(self, element):
+        data = {}
+        for elt in element:
+            tag = elt.tag.lower()
+            if tag == 'physical_drive':
+                tag += 's'
+                if tag not in data:
+                    data[tag] = []
+                data[tag].append(self._element_children_to_dict(elt))
+            else:
+                data[tag] = elt.get('VALUE')
+        return data
+
     def get_fw_version(self):
         """Get the iLO firmware version"""
         return self._info_tag('RIB_INFO', 'GET_FW_VERSION')
@@ -746,7 +855,7 @@ class Ilo(object):
 
     def get_ilo_event_log(self):
         """Get the full iLO event log"""
-        return self._info_tag2('RIB_INFO', 'GET_EVENT_LOG', 'EVENT_LOG')
+        return self._info_tag('RIB_INFO', 'GET_EVENT_LOG', 'EVENT_LOG')
 
     def get_language(self):
         """Get the default language set"""
@@ -770,12 +879,18 @@ class Ilo(object):
         def process(data):
             if 'device' in data['boot_type']:
                 data['boot_type'] = data['boot_type']['device']
-            return data
+            return data['boot_type'].lower()
         return self._info_tag('SERVER_INFO', 'GET_ONE_TIME_BOOT', ('ONE_TIME_BOOT', 'GET_ONE_TIME_BOOT'), process=process)
 
     def get_persistent_boot(self):
         """Get the boot order of the host"""
-        return self._info_tag('SERVER_INFO', 'GET_PERSISTENT_BOOT', 'PERSISTENT_BOOT')
+        def process(data):
+            if isinstance(data, dict):
+                data = data.items()
+                data.sort(key=lambda x: x[1])
+                return [x[0].lower() for x in data]
+            return [x.lower() for x in data]
+        return self._info_tag('SERVER_INFO', 'GET_PERSISTENT_BOOT', ('PERSISTENT_BOOT', 'GET_PERSISTENT_BOOT'), process=process)
 
     def get_power_cap(self):
         """Get the power cap setting"""
@@ -795,7 +910,11 @@ class Ilo(object):
 
     def get_server_event_log(self):
         """Get the IML log of the server"""
-        return self._info_tag2('SERVER_INFO', 'GET_EVENT_LOG', 'EVENT_LOG')
+        def process(data):
+            if isinstance(data, dict) and 'description' in data:
+                return []
+            return data
+        return self._info_tag('SERVER_INFO', 'GET_EVENT_LOG', 'EVENT_LOG', process=process)
 
     def get_server_name(self):
         """Get the name of the server this iLO is managing"""
@@ -841,6 +960,7 @@ class Ilo(object):
         """Import a signed SSL certificate"""
         return self._control_tag('RIB_INFO', 'IMPORT_CERTIFICATE', text=certificate)
 
+    # Broken in iLO3 < 1.55 for Administrator
     def import_ssh_key(self, user_login, ssh_key):
         """Imports an SSH key for the specified user. The value of ssh_key
            should be the content of an id_dsa.pub file"""
@@ -862,7 +982,6 @@ class Ilo(object):
     #    """Insert a virtual floppy"""
     #    return self._control_tag('RIB_INFO', 'INSERT_VIRTUAL_FLOPPY', attrib={'IMAGE_LOCATION': image_location})
 
-    @untested
     def delete_ssh_key(self, user_login):
         """Delete a users SSH key"""
         return self._control_tag('USER_INFO', 'MOD_USER', attrib={'USER_LOGIN': user_login}, elements=[etree.Element('DEL_USERS_SSH_KEY')])
@@ -1170,27 +1289,120 @@ class Ilo(object):
         return self._info_tag('RIB_INFO', 'GET_VF_STATUS')
 
 ###############################################################################
-### Testsuite, not to be run by end-users
-### Note: use only assertTrue/assertRaises for compatibility with older python
-### versions
-    def _test(self):
-        import unittest
-        this_ilo = self
-        class IloTest(unittest.TestCase):
-            @classmethod
-            def setUpClass(self):
-                sys.stdout.write("Identifying iLO version... ")
-                sys.stdout.flush()
-                self.ilo = this_ilo
-                res = self.ilo.get_fw_version()
-                print(res['management_processor'])
-                self.ilo_version = int(res['management_processor'][3:] or 1)
-                print("Running tests. This will take a few minutes")
+# Testsuite, safe to run on all iLO versions. Reports of failures of the
+# testsuite are very much appreciated. The testsuite by default makes no
+# changes to the iLO configuration.
+#
+# You can use _test_writes instead of _test to also test methods that make
+# changes to the iLO configuration. All changes should be undone, but no
+# guarantees are made.
+#
+# The writing tests also clear the iLO and server event log, which cannot be
+# undone. The iLO will also be reset at least once.  A firware upgrade will not
+# be attempted by the test.
+#
+# To run these tests: use hpilo_cli hostname_here _test To run only a subset of
+# the test: hpilo_cli hostname_here testname_here [another testname ...]
 
-            def test_get_all_users(self):
+    def _test(self, opts, tests):
+        import unittest
+        import time
+        this_ilo = self
+        sys.stdout.write("Identifying iLO version... ")
+        sys.stdout.flush()
+        res = self.get_fw_version()
+        print(res['management_processor'])
+        print("Running tests. This will take a few minutes")
+
+        class IloTest(unittest.TestCase):
+            ilo = this_ilo
+            do_write_tests = opts.do_write_tests
+            ilo_version = int(res['management_processor'][3:] or 1)
+
+            def test_non_ilo(self):
+                if self.ilo.protocol == ILO_LOCAL:
+                    return
+                def get_socket(*args):
+                    class FakeSocket(object):
+                        def __init__(self):
+                            self.data = b('Bogus data')
+
+                        def write(self, data):
+                            pass
+
+                        def read(self):
+                            d = self.data
+                            self.data = b('')
+                            return d
+
+                        def shutdown(self, what):
+                            pass
+
+                        def close(self):
+                            pass
+
+                    return FakeSocket()
+                real_get_socket, self.ilo._get_socket = self.ilo._get_socket, get_socket
+
+                try:
+                    self.assertRaises(IloError, self.ilo.get_fw_version)
+                finally:
+                    self.ilo._get_socket = real_get_socket
+
+            def test_connect_error(self):
+                port, self.ilo.port = self.ilo.port, 1
+                hponcfg, self.ilo.hponcfg = self.ilo.hponcfg, '/sbin/does/not/exist'
+                try:
+                    self.assertRaises(IloCommunicationError, self.ilo.get_fw_version)
+                finally:
+                    self.ilo.port = port
+                    self.ilo.hponcfg = hponcfg
+
+            def test_wrong_method(self):
+                if self.ilo_version < 4:
+                    method = self.ilo.get_ahs_status
+                else:
+                    method = self.ilo.get_cert_subject_info
+
+                self.assertRaises(IloError, method)
+
+            def test_users(self):
                 users = self.ilo.get_all_users()
                 self.assertTrue(isinstance(users, list))
                 self.assertTrue(self.ilo.login in users)
+                users = self.ilo.get_all_user_info()
+                self.assertTrue(isinstance(users, dict))
+                self.assertTrue(self.ilo.login in users)
+                self.assertTrue(users[self.ilo.login]['admin_priv'])
+                res = self.ilo.get_user(self.ilo.login)
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue(res['user_login'] == self.ilo.login)
+                self.assertTrue('admin_priv' in res)
+                if not self.do_write_tests:
+                    return
+                if 'dennis' in users:
+                    warnings.warn('User dennis exists, not testing user manipulation', IloTestWarning)
+                    return
+                try:
+                    self.ilo.add_user('dennis', 'Dennis Kaarsemaker', 'Password123')
+                    users = self.ilo.get_all_users()
+                    self.assertTrue('dennis' in users)
+                    key = 'ssh-dss AAAAB3NzaC1kc3MAAACBAIpNY5fvLSS3MCjGNKjuWHrFGR5J6vLqdqIrXttTz7o6GWtmyxcC0Mlp2c/h1bMfvUiKDvDp+5T7SGo/2R+aXLaPwYtm6eBPEBU2CgVTnpeVELDeaJ/tr0kTL/PKMHZDFgT9c7/hOiWr4amlGvuxs60MP/xs4jWaxLxabhjiRoCLAAAAFQChDEFySo74rpPNNWfvJHgiylTbRQAAAIEAgo8UQqXP7gMTAUdHTqlzoTnj3loc4ZTnf3W6jr25cs5XaXNnRtadfw0G4VWaS/uDyNhsq/o2nFrhWTwAvojWSe4C5MDdGGerktL1ZY/QfoxB0d7aK/dlHd1iOVpGahCqyzmhEDmEnq6TWd6cBVHNVcryLEJVVtaf8QmJlwS+XkIAAACAJGnuO6ZJ1S2AMOY1uOpov/srTyuu6PxtcnHsHA5wNoNQFcYElnDndJUfMAPi0vzODntHoiOGdrX3RcjxSAB5lAgNZwFnwGWoAa8UIQlX+GwDYAIk+8G36tmHRgtl7xJlFqs9W6BhrJEmfL4ubWCPXl/yMDrrLnMQuV3Mg0DNVSg= Ilo test key'
+                    #self.ilo.import_ssh_key('dennis', key)
+                    if self.ilo_version == 2:
+                        self.ilo.delete_ssh_key('dennis')
+                    self.ilo.mod_user('dennis', user_name='Dennis Kaarsemaker Test')
+                    res = self.ilo.get_user('dennis')
+                    self.assertTrue(res['user_name'] == 'Dennis Kaarsemaker Test')
+                    self.ilo.delete_user('dennis')
+                    users = self.ilo.get_all_users()
+                    self.assertTrue('dennis' not in users)
+                except:
+                    # Clean up after ourselves
+                    users = self.ilo.get_all_users()
+                    if 'dennis' in users:
+                        self.ilo.delete_user('dennis')
+                    raise
 
             def test_get_embedded_health(self):
                 if self.ilo_version < 2:
@@ -1198,12 +1410,24 @@ class Ilo(object):
                 res = self.ilo.get_embedded_health()
                 self.assertTrue(isinstance(res, dict))
                 self.assertTrue('temperature' in res)
+                self.assertTrue('fans' in res)
+                self.assertTrue('fans' in res['health_at_a_glance'])
+
+                return # FIXME - need to implement this first
+                if self.ilo_version >= 3:
+                    self.assertTrue('drives' in res)
+                if self.ilo_version >= 4:
+                    self.assertTrue('storage' in res)
+                    self.assertTrue('nic_information' in res)
+                    self.assertTrue('memory' in res)
+                    self.assertTrue('firmware_information' in res)
 
             def test_get_cert_subject_info(self):
-                if self.ilo_version < 2:
+                if self.ilo_version != 2:
                     return
                 res = self.ilo.get_cert_subject_info()
                 self.assertTrue(isinstance(res, dict))
+                self.assertTrue('csr_subject_country' in res)
 
             def test_get_host_data(self):
                 res = self.ilo.get_host_data()
@@ -1214,36 +1438,207 @@ class Ilo(object):
                 self.assertTrue(self.issubset(res, res2))
                 self.assertTrue('b64_data' in res[0])
 
-            def test_get_host_power_status(self):
-                res = self.ilo.get_host_power_status()
-                self.assertTrue(isinstance(res, dict))
-                self.assertTrue('host_power_saver' in res)
-
-            def test_get_one_time_boot(self):
+            def test_boot(self):
                 if self.ilo_version < 2:
                     return
-                res = self.ilo.get_one_time_boot()
+                values = self.ilo.get_persistent_boot()
+                self.assertTrue(isinstance(values, list))
+                self.assertTrue('hdd' in values)
+                boot_type = self.ilo.get_one_time_boot()
+                self.assertTrue(isinstance(boot_type, basestring))
+                self.assertTrue(boot_type in values + ['normal'])
+                if self.do_write_tests:
+                    if boot_type in values:
+                        values.remove(boot_type)
+                    try:
+                        self.ilo.set_one_time_boot(values[0])
+                        res = self.ilo.get_one_time_boot()
+                        self.assertTrue(res == values[0])
+                    finally:
+                        self.ilo.set_one_time_boot(boot_type)
+
+            def test_uid(self):
+                status = self.ilo.get_uid_status()
+                self.assertTrue(status in ('ON', 'OFF'))
+                if not self.do_write_tests:
+                    return
+                if status == 'OFF':
+                    self.ilo.uid_control(uid='Yes')
+                    self.assertTrue(self.ilo.get_uid_status() == 'ON')
+                    self.ilo.uid_control(uid='No')
+                    self.assertTrue(self.ilo.get_uid_status() == 'OFF')
+                else:
+                    self.ilo.uid_control(uid='No')
+                    self.assertTrue(self.ilo.get_uid_status() == 'OFF')
+                    self.ilo.uid_control(uid='Yes')
+                    self.assertTrue(self.ilo.get_uid_status() == 'ON')
+
+            def test_get_dir_config(self):
+                if self.ilo_version < 2:
+                    return
+                res = self.ilo.get_dir_config()
                 self.assertTrue(isinstance(res, dict))
-                self.assertTrue('boot_type' in res)
+                self.assertTrue('dir_user_context_1' in res)
 
-            def test_get_server_power_on_time(self):
-                res
+            def test_delayed(self):
+                if self.ilo_version < 2:
+                    return
+                uid = {'ON': 'Yes', 'OFF': 'No'}[self.ilo.get_uid_status()]
+                self.ilo.delayed = True
+                try:
+                    self.ilo.get_all_users()                # Getter
+                    self.ilo.uid_control(uid=uid)           # Control tag
+                    self.ilo.certificate_signing_request()  # Control tag that returns something
+                    self.ilo.get_all_user_info()            # Getter
+                    res = self.ilo.call_delayed()
+                finally:
+                    self.ilo.delayed = False
+                self.assertTrue(isinstance(res, list))
+                self.assertTrue(len(res) == 3)
+                self.assertTrue(self.ilo.login in res[0])
+                self.assertTrue('-----' in res[1])
+                self.assertTrue(self.ilo.login in res[2])
 
-            # Tests to write:
-            #
-            # get_server_power_on_time
-            # get_sso_settings
-            # Auth error
-            # Connect error
-            # Wrong method for ilo version error
-            # uid_control
+                # And are we now in non-delayed mode and usable?
+                res = self.ilo.get_all_users()
+                self.assertTrue(isinstance(res, list))
+                self.assertTrue(self.ilo.login in res)
 
-            if os.environ.get('HPILO_TEST_CHANGES', None) == 'OK':
-                pass
-                # Tests that make changes:
-                # activate_license
-                # add_user/delete_user/import_ssh_key for user/delete_ssh_key
-                # mod_global_settings ssh_port 2222 / 22
+            # get_server_auto_pwr
+            def test_power(self):
+                res = self.ilo.get_host_power_status()
+                self.assertTrue(res in ('ON', 'OFF'))
+                if self.ilo_version < 2:
+                    return
+                res = self.ilo.get_host_power_saver_status()
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue('host_power_saver' in res)
+                res = self.ilo.get_server_power_on_time()
+                self.assertTrue(isinstance(res, int))
+                res = self.ilo.get_power_cap()
+                self.assertTrue(res in ('ON', 'OFF'))
+                res = self.ilo.get_power_readings()
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue('average_power_reading' in res)
+                res = self.ilo.get_pwreg()
+                self.assertTrue(isinstance(res, dict))
+                res = self.ilo.get_server_auto_pwr()
+                self.assertTrue(isinstance(res, basestring))
+                if not self.do_write_tests:
+                    return
+                self.ilo.clear_server_power_on_time()
+                res = self.ilo.get_server_power_on_time()
+                self.assertTrue(isinstance(res, int))
+                self.assertTrue(res < 2)
+
+            def test_get_sso_settings(self):
+                if self.ilo_version < 2:
+                    return
+                res = self.ilo.get_sso_settings()
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue('administrator_role' in res)
+                self.assertTrue(isinstance(res['administrator_role'], dict))
+                self.assertTrue(res['administrator_role']['reset_server_priv'] == True)
+
+            def test_global_settings(self):
+                res = self.ilo.get_global_settings()
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue(res['https_port'] == self.ilo.port)
+                if self.do_write_tests:
+                    min_password = res['min_password']
+                    self.ilo.mod_global_settings(min_password=8)
+                    res = self.ilo.get_global_settings()
+                    self.ilo.mod_global_settings(min_password=min_password)
+                    self.assertTrue(res['min_password'] == 8)
+
+            def test_ilo_event_log(self):
+                res = self.ilo.get_ilo_event_log()
+                self.assertTrue(isinstance(res, list))
+                self.assertTrue(isinstance(res[0], dict))
+                if not self.do_write_tests:
+                    self.ilo.clear_ilo_event_log()
+                    res = self.ilo.get_ilo_event_log()
+                    self.assertTrue(len(res) == 3)
+                    self.assertTrue(res[0]['description'].startswith('Event log cleared'))
+                    self.assertTrue(res[2]['description'].startswith('XML logout'))
+
+            def test_languages(self):
+                if self.ilo_version < 3:
+                    return
+                res = self.ilo.get_all_languages()
+                self.assertTrue(isinstance(res, dict))
+                res = self.ilo.get_language()
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue('lang_id' in res)
+                if self.do_write_tests:
+                    self.ilo.set_language(res['lang_id'])
+
+            def test_get_host_pwr_micro_ver(self):
+                if self.ilo_version < 2:
+                    return
+                res = self.ilo.get_host_pwr_micro_ver()
+                self.assertTrue(isinstance(res, str))
+                self.assertTrue(res != "")
+
+            def test_get_oa_info(self):
+                if self.ilo_version < 2:
+                    return
+                try:
+                    res = self.ilo.get_oa_info()
+                except IloError:
+                    e = sys.exc_info()[1]
+                    self.assertTrue('not a rack server' in str(e).lower())
+                else:
+                    self.assertTrue('rack' in res)
+
+            def test_get_vm_status(self):
+                res = self.ilo.get_vm_status()
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue('device' in res)
+
+            def test_network_settings(self):
+                res = self.ilo.get_network_settings()
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue('sec_wins_server' in res)
+                if self.do_write_tests and not res['dhcp_wins_server']:
+                    old_wins_server = res['sec_wins_server']
+                    try:
+                        self.ilo.mod_network_settings(sec_wins_server='127.1.2.3')
+                        # This resets the iLO board, wait for it to come back
+                        time.sleep(70)
+                        res = self.ilo.get_network_settings()
+                        self.assertTrue(res['sec_wins_server'] == '127.1.2.3')
+                    finally:
+                        self.ilo.mod_network_settings(sec_wins_server=old_wins_server)
+                        # This resets the iLO board, wait for it to come back
+                        time.sleep(70)
+
+            def test_get_server_event_log(self):
+                res = self.ilo.get_server_event_log()
+                self.assertTrue(isinstance(res, list))
+                if res:
+                    self.assertTrue(isinstance(res[0], dict))
+                    self.assertTrue('class' in res[0])
+
+            def test_snmp(self):
+                res = self.ilo.get_snmp_im_settings()
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue('snmp_address_3' in res)
+                if self.do_write_tests:
+                    old = res['snmp_address_3']
+                    try:
+                        self.ilo.mod_snmp_im_settings(snmp_address_3='127.2.3.4')
+                        res = self.ilo.get_snmp_im_settings()
+                        self.assertTrue(res['snmp_address_3'] == '127.2.3.4')
+                    finally:
+                        self.ilo.mod_snmp_im_settings(snmp_address_3=old)
+
+            def test_get_twofactor_settings(self):
+                if self.ilo_version != 2:
+                    return
+                res = self.ilo.get_twofactor_settings()
+                self.assertTrue(isinstance(res, dict))
+                self.assertTrue('auth_twofactor_enable' in res)
 
             def issubset(self, a, b):
                 for elt in a:
@@ -1251,5 +1646,12 @@ class Ilo(object):
                         return False
                 return  True
 
+        # Limit tests if requested
+        # FIXME: there has to be a better way than this
+        if tests:
+            for attr in list(IloTest.__dict__.keys()):
+                if attr.startswith('test_') and attr not in tests and attr[5:] not in tests:
+                    delattr(IloTest, attr)
         self.IloTest = IloTest
-        unittest.main(self, argv=[sys.argv[0], 'IloTest'])
+        runner = unittest.TextTestRunner(verbosity=2)
+        unittest.main(self, argv=[sys.argv[0], 'IloTest'], testRunner=runner)
